// Copyright 1993 by Jon Dart. All Rights Reserved.

#ifndef _BOOK_ENTRY_H
#define _BOOK_ENTRY_H

#include "types.h"
#include "hashable.h"
#include "pool.h"
#include <malloc.h>

class Book_Entry : public Hashable
{
     // defines one entry in the opening book's hash table.

public:

     Book_Entry()
       :my_hash_code(0),frequency(0),move_index(0),flags(0),
        winloss(0),learn_score(0.0)
     {
     }

     Book_Entry( hash_t hc, byte rec, byte mv_indx,
	         int last = 0);

     Book_Entry(const Book_Entry &be) 
       :
       my_hash_code(be.my_hash_code),
       frequency(be.frequency),
       move_index(be.move_index),
       flags(be.flags),
       winloss(be.winloss),
       learn_score(be.learn_score),
       flags2(be.flags2)
     {
     }


     void init( hash_t hc, uint16 freq, byte flags, byte mv_indx);
 
     // two entries are equivalent if they have the same hash code bytes.
     virtual bool is_equal( const Hashable &c) const;
	    
     hash_t hash_code() const
     {
        return my_hash_code;
     }
     
     byte get_move_index() const
     { 
	return move_index;
     }
     
     int get_winloss() const
     {
        return (int)winloss;
     }
     
     void set_winloss(int w)
     {
        winloss = w;
     }
     
     uint32 get_frequency() const;
     
     void set_frequency(uint32 f);
     
     int is_last() const;

     void set_last(int b = 1);

     unsigned get_recommend() const;

     void set_basic(int b = 1);

     int is_basic() const;

     int get_learned_result()  const;

     void update_learned_result(int diff);

     hash_t my_hash_code;
     byte frequency;
     byte move_index;
     byte flags;
     signed char winloss;
     float learn_score;
     byte flags2;
     static Pool allocator;
};

#endif
