// Copyright 1994 by Jon Dart. All Rights Reserved.
// $Revision: 1.1.1.1 $
//
#include "pool.h"

Pool::Pool()
{
   memChain = freeChunks = NULL;
   nextMem = NULL;
   freeChain = NULL;
}

Pool::~Pool()
{
}

void *Pool::allocate( size_t size )
{
   if (!freeChain)
   {
      // allow room for "next" pointer
      size += sizeof(void*);
      // align it
      if (size % 4) size += (4 - (size % 4));
      if (!memChain || 
          size >= (int)BLOCK_SIZE - (nextMem - memChain->blk->mem))
      {
         chunk *newChunk;
         if (freeChunks)
         {
            newChunk = freeChunks;
            freeChunks = freeChunks->next;
         }
         else
         {
            newChunk = new chunk;
            newChunk->blk = new block;
         }
         newChunk->next = memChain;
         memChain = newChunk;
         nextMem = newChunk->blk->mem;
      }

      alloc_ptr p = (alloc_ptr)nextMem;
      nextMem += size;
      p->next = NULL;
      return &(p->mem);
   }
   else
   {
      alloc_ptr aFreeAlloc = freeChain;
      freeChain = aFreeAlloc->next;
      return &(aFreeAlloc->mem);
   }
}

void Pool::free( void *p )
{
   alloc_ptr q = (alloc_ptr)((char*)p-sizeof(void*));
   q->next = freeChain;
   freeChain = q;
}

void Pool::freeAll(int final)
{
   if (final)
   {
      // delete free chunks
      chunk *c = freeChunks;
      while (c)
      {
         chunk *nxt = c->next;
         delete [] c->blk;
         delete c;
         c = nxt;
      }
   }
   chunk *c = memChain;
   while (c)
   {
      chunk *nxt = c->next;
      if (final)
      {
         delete [] c->blk;
         delete c;
      }
      else
      {
         c->next = freeChunks;
         freeChunks = c;
      }
      c = nxt;
   }
   memChain = NULL;
   nextMem = NULL;
   freeChain = NULL;
   if (final)
      freeChunks = NULL;
}
