// Copyright 1994, 1997 by Jon Dart.  All Rights Reserved.

#ifndef _SQUARE_H
#define _SQUARE_H

#include "types.h"
#include "color.h"

// Square used to be a class but now it is typedef'd to an int,
// for efficiency reasons. This header defines several convenience
// macros and functions for dealing with squares.

typedef unsigned int Square;

enum {InvalidSquare = 127};

extern const int Files[64];
extern const int Ranks[64];
extern const int Edge[64];
extern const int Colors[64];
extern const int Center[64];
extern const int Corners[64];

#define Square( file, rank, side ) \
  ((side == Black) ? ((rank-1)*8 + file - 1) : (55 - (rank-1)*8 + file))
    
// returns "rank" from the perspective of "side".  1 = 1st
// rank for side, 8 = last rank.	    
#define Rank(square, side ) ((side == Black) ? Ranks[square] : 9-Ranks[square])
#define WhiteRank(square) 9-Ranks[square]
#define BlackRank(square) Ranks[square]

// returns file.  Queen rook file = 1, King rook file = 8.
#define File(square) Files[square]
	    
#define OnBoard(square) ((square & ~0x3f) == 0)

#define OnEdge(square) Edge[square]

#define InCenter(square) Center[square]

#define InCorner(square) Corners[square]
    
#define IsInvalid(square) (square == InvalidSquare)
    
#define SquareColor(square) ((ColorType)Colors[square])
    
// Parse a square value in algebraic notation (e.g. "g7") and
// return a corresponding Square type.
extern Square SquareValue(const char *p);
    
extern char FileImage(Square sq);

extern char RankImage(Square sq);

// algebraic notation to square mapping.

#define A8 0
#define B8 1
#define C8 2
#define D8 3
#define E8 4
#define F8 5
#define G8 6
#define H8 7
#define A7 8
#define B7 9
#define C7 10
#define D7 11
#define E7 12
#define F7 13
#define G7 14
#define H7 15
#define A6 16
#define B6 17
#define C6 18
#define D6 19
#define E6 20
#define F6 21
#define G6 22
#define H6 23
#define A5 24
#define B5 25
#define C5 26
#define D5 27
#define E5 28
#define F5 29
#define G5 30
#define H5 31
#define A4 32
#define B4 33
#define C4 34
#define D4 35
#define E4 36
#define F4 37
#define G4 38
#define H4 39
#define A3 40
#define B3 41
#define C3 42
#define D3 43
#define E3 44
#define F3 45
#define G3 46
#define H3 47
#define A2 48
#define B2 49
#define C2 50
#define D2 51
#define E2 52
#define F2 53
#define G2 54
#define H2 55
#define A1 56
#define B1 57
#define C1 58
#define D1 59
#define E1 60
#define F1 61
#define G1 62
#define H1 63

#define AFILE 1
#define BFILE 2
#define CFILE 3
#define DFILE 4
#define EFILE 5
#define FFILE 6
#define GFILE 7
#define HFILE 8

#endif

